@@ -7,11 +7,11 @@
 ## http://search.cpan.org/dist/Devel-NYTProf/
 ##
 ###########################################################
-## $Id: NYTProf.pm 1157 2010-03-09 10:35:10Z tim.bunce $
+## $Id: NYTProf.pm 1300 2010-06-16 22:18:00Z tim.bunce@gmail.com $
 ###########################################################
 package Devel::NYTProf;
 
-our $VERSION = '3.11';
+our $VERSION = '4.02';
 
 package    # hide the package from the PAUSE indexer
     DB;
@@ -38,7 +38,7 @@ if ($use_db_sub) {                     # install DB::DB sub
 # DB::sub shouldn't be called, but needs to exist for perl <5.8.7 (<perl@24265)
 # Could be called in obscure cases, e.g. if "perl -d" (not -d:NYTProf)
 # was used with Devel::NYTProf loaded some other way
-sub sub { die "DB::sub called unexpectly" }
+sub sub { die "DB::sub called unexpectedly" }
 
 sub CLONE { DB::disable_profiler }
 
@@ -50,7 +50,7 @@ __END__
 
 =head1 NAME
 
-Devel::NYTProf - Powerful feature-rich perl source code profiler
+Devel::NYTProf - Powerful fast feature-rich perl source code profiler
 
 =head1 SYNOPSIS
 
@@ -65,7 +65,7 @@ Devel::NYTProf - Powerful feature-rich perl source code profiler
 
 =head1 DESCRIPTION
 
-Devel::NYTProf is a powerful feature-rich perl source code profiler.
+Devel::NYTProf is a powerful, fast, feature-rich perl source code profiler.
 
 =over
 
@@ -107,7 +107,7 @@ Uses novel techniques for efficient profiling
 
 =item *
 
-Sub-microsecond (100ns) resolution on systems with clock_gettime()
+Sub-microsecond (100ns) resolution on supported systems
 
 =item *
 
@@ -131,6 +131,10 @@ Generates richly annotated and cross-linked html reports
 
 =item *
 
+Captures source code, including string evals, for stable results
+
+=item *
+
 Trivial to use with mod_perl - add one line to httpd.conf
 
 =item *
@@ -389,21 +393,27 @@ The default 'opcode redirection' technique can't profile subroutines that were
 compiled before NYTProf was loaded. So using use_db_sub=1 can be useful in
 cases where you can't load the profiler early in the life of the application.
 
-=head2 savesrc=1
+Another side effect of C<use_db_sub=1> is that it enables recording of the
+source code of the C<< perl -e '...' >> and C<< perl - >> input for old
+versions of perl. See also L</savesrc=0>.
 
-Save a copy of all source code into the profile data file. This makes the file
-self-contained, so the reporting tools no longer depend on having the original
-source code files available. So it also insulates you from later changes to
-those files that would normally make the reports out of sync with the data.
+=head2 savesrc=0
 
-By default NYTProf saved some source code: the arguments to the C<perl -e>
-option, the script fed to perl via STDIN when using C<perl ->, and the source
-code of string evals. (Currently string eval  source code isn't available in
-the reports. Patches welcome.)
+Disable the saving of source code.
 
-If you're using perl 5.10.0 or 5.8.8 (or earlier) then you need to also enable
-the C<use_db_sub=1> option otherwise perl doesn't make the source code
-available to NYTProf. Perl 5.8.9 and 5.10.1+ don't require that.
+By default NYTProf saves a copy of all source code into the profile data file.
+This makes the file self-contained, so the reporting tools no longer depend on
+having the unmodified source code files available.
+
+With C<savesrc=0> some source code is still saved: the arguments to the
+C<perl -e> option, the script fed to perl via STDIN when using C<perl ->,
+and the source code of string evals.
+
+Saving the source code of string evals requires perl version 5.8.9+, 5.10.1+,
+or 5.12 or later.
+
+Saving the source code of the C<< perl -e '...' >> or C<< perl - >> input
+requires either a recent perl version, as above, or setting the L</use_db_sub=1> option.
 
 =head2 slowops=N
 
@@ -488,6 +498,16 @@ separated by commas, as the value of the option (case is not significant):
 
     sigexit=int,hup
 
+=head2 posix_exit=1
+
+The NYTProf subroutine profiler normally detects calls to C<POSIX::_exit()>
+(which exits the process without running END blocks) and automatically calls
+C<DB::finish_profile()> for you, so NYTProf 'just works'.
+
+When using the C<subs=0> option to disable the subroutine profiler the
+C<posix_exit> option can be used to tell NYTProf to take other steps to arrange
+for C<DB::finish_profile()> to be called before C<POSIX::_exit()>.
+
 =head2 forkdepth=N
 
 When a perl process that is being profiled executes a fork() the child process
@@ -544,7 +564,9 @@ closed.  Calling DB::disable_profile() doesn't do that.  To make a profile file
 usable before the profiled application has completed you can call
 DB::finish_profile(). Alternatively you could call DB::enable_profile($newfile).
 
-=head2 DB::disable_profile()
+=head2 disable_profile
+
+  DB::disable_profile()
 
 Stops collection of profile data.
 
@@ -552,18 +574,24 @@ Subroutine calls which were made while profiling was enabled and are still on
 the call stack (have not yet exited) will still have their profile data
 collected when they exit.
 
-=head2 DB::enable_profile($newfile)
+=head2 enable_profile
+
+  DB::enable_profile($newfile)
 
 Enables collection of profile data. If $newfile is true the profile data will be
 written to $newfile (after completing and closing the previous file, if any).
 If $newfile already exists it will be deleted first.
 
-=head2 DB::finish_profile()
+=head2 finish_profile
+
+  DB::finish_profile()
 
 Calls DB::disable_profile(), then completes the profile data file by writing
 subroutine profile data, and then closes the file. The in memory subroutine
 profile data is then discarded.
 
+Normally NYTProf arranges to call finish_profile() for you via an END block.
+
 =head1 DATA COLLECTION AND INTERPRETATION
 
 NYTProf tries very hard to gather accurate information.  The nature of the
@@ -581,6 +609,76 @@ Perl 5.12 will hopefully also fix an inaccuracy in the timing of the last
 statement and the condition clause of some kinds of loops:
 L<http://rt.perl.org/rt3/Ticket/Display.html?id=60954>
 
+=head2 eval $string
+
+Perl treats each execution of a string eval (C<eval $string;> not C<eval { ...  }>)
+as a distinct file, so NYTProf does as well. The 'files' are given names with
+this structure:
+
+	(eval $sequence)[$filename:$line]
+
+for example "C<(eval 93)[/foo/bar.pm:42]>" would be the name given to the
+93rd execution of a string eval by that process and, in this case, the 93rd
+eval happened to be one at line 42 of "/foo/bar.pm".
+
+Nested string evals can give rise to file names like
+
+	(eval 1047)[(eval 93)[/foo/bar.pm:42]:17]
+
+=head3 Merging Evals
+
+Some applications execute a great many string eval statements. If NYTProf generated
+a report page for each one it would not only slow report generation but also
+make the overall report less useful by scattering performance data too widely.
+On the other hand, being able to see the actual source code executed by an
+eval, along with the timing details, is often I<very> useful.
+
+To try to balance these conflicting needs, NYTProf currently I<merges
+uninteresting string eval siblings>.
+
+What does that mean? Well, for each source code line that executed any string
+evals, NYTProf first gathers the corresponding eval 'files' for that line
+(known as the 'siblings') into groups keyed by distinct source code.
+
+Then, for each of those groups of siblings, NYTProf will 'merge' a group
+that shares the same source code and doesn't execute any string evals itself.
+Merging means to pick one sibling as the survivor and merge and delete all
+the data from the others into it.
+
+If there are a large number of sibling groups then the data for all of them are
+merged into one regardless.
+
+The report annotations will indicate when evals have been merged together.
+
+=head3 Merging Anonymous Subroutines
+
+Anonymous subroutines defined within string evals have names like this:
+
+	main::__ANON__[(eval 75)[/foo/bar.pm:42]:12]
+
+That anonymous subroutine was defined on line 12 of the source code executed by
+the string eval on line 42 of F</foo/bar.pm>. That was the 75th string eval
+executed by the program.
+
+Anonymous subroutines I<defined on the same line of sibling evals that get
+merged> are also merged. That is, the profile information is merged into
+one and the others are discarded.
+
+=head3 Timing
+
+Care should be taken when interpreting the report annotations associated with a
+string eval statement.  Normally the report annotations embedded into the
+source code related to timings from the I<subroutine> profiler. This isn't
+(currently) true of annotations for string eval statements.
+
+This makes a significant different if the eval defines any subroutines that get
+called I<after> the eval has returned. Because the time shown for a string eval
+is based on the I<statement> times it will include time spent executing
+statements within the subs defined by the eval.
+
+In future NYTProf may involve the subroutine profiler in timings evals and so
+be able to avoid this issue.
+
 =head2 Calls from XSUBs and Opcodes
 
 Calls record the current filename and line number of the perl code at the time
@@ -610,7 +708,7 @@ You can reduce the cost of profiling by adjusting some options. The trade-off
 is reduced detail and/or accuracy in reports.
 
 If you don't need statement-level profiling then you can disable it via L</stmts=0>.
-If you do need it but don't mind loosing block-level timings then set L</blocks=0>.
+If you do want it but don't mind loosing block-level timings then set L</blocks=0>.
 If you want need still more speed then set L</leave=0> to disable the
 adjustments made for statements that 'leave' the current control flow (doing
 this I<will> make timings for some kinds of statements less accurate).
@@ -618,9 +716,14 @@ this I<will> make timings for some kinds of statements less accurate).
 If you don't need subroutine profiling then you can disable it via L</subs=0>.
 If you do need it but don't need timings for perl opcodes then set L</slowops=0>.
 
+Generally speaking, setting blocks=0 and slowops=0 will give you a useful boost
+with the least loss of detail.
+
 Another approach is to only enable NYTProf in the sections of code that
 interest you. See L</RUN-TIME CONTROL OF PROFILING> for more details.
 
+To speed up L<nytprofhtml> try using the --minimal (-m) option.
+
 =head1 REPORTS
 
 The L<Devel::NYTProf::Data> module provides a low-level interface for loading
@@ -633,6 +736,12 @@ perl. (Though the module is in a state of flux and may be deprecated soon.)
 Included in the bin directory of this distribution are some scripts which
 turn the raw profile data into more useful formats:
 
+=head2 nytprofhtml
+
+Creates attractive, richly annotated, and fully cross-linked html
+reports (including statistics, source code and color highlighting).
+This is the main report generation tool for NYTProf.
+
 =head2 nytprofcsv
 
 Creates comma delimited profile reports. Old and limited.
@@ -642,11 +751,6 @@ Creates comma delimited profile reports. Old and limited.
 Translates a profile into a format that can be loaded into KCachegrind
 L<http://kcachegrind.sourceforge.net>
 
-=head2 nytprofhtml
-
-Creates attractive, richly annotated, and fully cross-linked html
-reports (including statistics, source code and color highlighting).
-
 =head2 nytprofmerge
 
 Reads multiple profile data files and writes out a new file containing the merged profile data.
@@ -786,22 +890,19 @@ This section lists other clock sources that NYTProf may use.
 =head3 gettimeofday
 
 This is the traditional high resolution time of day interface for most
-unix-like systems. It's used on platforms like Mac OS X which don't
-(yet) support C<clock_gettime()>.
+unix-like systems.  With this clock NYTProf outputs timings as a count of 1
+microsecond ticks.
 
-With this clock NYTProf outputs timings as a count of 1 microsecond ticks.
+=head3 mach_absolute_time
 
-=for comment re high resolution timing for OS X:
-http://developer.apple.com/qa/qa2004/qa1398.html
-http://www.macresearch.org/tutorial_performance_and_time
-http://cocoasamurai.blogspot.com/2006/12/tip-when-you-must-be-precise-be-mach.html
-http://boredzo.org/blog/archives/2006-11-26/how-to-use-mach-clocks
+On Mac OS X the mach_absolute_time() function is used. With this clock NYTProf
+outputs timings as a count of 100 nanosecond ticks.
 
 =head3 Time::HiRes
 
-On systems which don't support C<clock_gettime()> or C<gettimeofday()>
-NYTProf falls back to using the L<Time::HiRes> module.
-With this clock NYTProf outputs timings as a count of 1 microsecond ticks.
+On systems which don't support other clocks, NYTProf falls back to using the
+L<Time::HiRes> module.  With this clock NYTProf outputs timings as a count of 1
+microsecond ticks.
 
 =head2 Clock References
 
@@ -901,7 +1002,7 @@ but only at a very low 1/100th of a second resolution.
 
 =head1 BUGS
 
-Possibly.
+Possibly. All complex software has bugs. Let me know if you find one.
 
 =head1 SEE ALSO
 
@@ -915,7 +1016,7 @@ L<http://blog.timbunce.org/2008/07/16/nytprof-v2-the-background-story/>.
 
 Mailing list and discussion at L<http://groups.google.com/group/develnytprof-dev>
 
-Blog posts L<http://blog.timbunce.org/tag/nytprof/> and L<http://technorati.com/search/nytprof>
+Blog posts L<http://blog.timbunce.org/tag/nytprof/>
 
 Public SVN Repository and hacking instructions at L<http://code.google.com/p/perl-devel-nytprof/>
 
@@ -929,6 +1030,33 @@ it's very likely to be deprecated in a future release).
 L<Devel::NYTProf::ReadStream> is the module that lets you read a profile data
 file as a stream of chunks of data.
 
+=head1 TROUBLESHOOTING
+
+=head2 "Profile data incomplete, ..."
+
+This error message means the file doesn't contain all the expected data.
+That may be because it was truncated (perhaps the filesystem was full) or,
+more commonly, because the all the expected data hasn't been written.
+
+NYTProf writes some important data to the data file when I<finishing> profiling.
+If you read the file before the profiling has finished you'll get this error.
+
+If the process being profiled is still running you'll need to wait until it
+exits cleanly (runs C<END> blocks or L</finish_profile> is called explicitly).
+
+If the process being profiled has exited then it's likely that it met with a
+sudden and unnatural death that didn't give NYTProf a chance to finish the profile.
+If the sudden death was due to a signal then L</sigexit=1> may help.
+If the sudden death was due to calling C<POSIX::_exit($status)> then you'll
+need to call L</finish_profile> before calling C<POSIX::_exit>.
+
+=head2 Some files don't have profile information
+
+This is usually due to NYTProf being loaded after the other files, for example
+
+If you can't alter the command line to add "C<-d:NYTProf>" you could try using
+the C<PERL5OPT> environment variable. See L</PROFILING>.
+
 =head1 AUTHORS AND CONTRIBUTORS
 
 B<Tim Bunce> (L<http://www.tim.bunce.name> and L<http://blog.timbunce.org>)
@@ -950,7 +1078,7 @@ For more details see L</HISTORY> below.
 =head1 COPYRIGHT AND LICENSE
 
   Copyright (C) 2008 by Adam Kaplan and The New York Times Company.
-  Copyright (C) 2008, 2009 by Tim Bunce, Ireland.
+  Copyright (C) 2008-2010 by Tim Bunce, Ireland.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
@@ -1009,7 +1137,8 @@ cross-linked reports.
 
 Steve Peters came on board along the way with patches for portability and to
 keep NYTProf working with the latest development perl versions. Nicholas Clark
-added zip compression. Jan Dubois contributed Windows support.
+added zip compression, many optimizations, and C<nytprofmerge>.
+Jan Dubois contributed Windows support.
 
 Adam's work is sponsored by The New York Times Co. L<http://open.nytimes.com>.
 Tim's work was partly sponsored by Shopzilla L<http://www.shopzilla.com> during 2008.
